let doc = """
Renamer.

Usage:
  renamer <path>... [--season=<se>] [--bash] [--log] [--recurse]
  renamer (-h | --help)
  renamer [--version]

Options:
  -h, --help     Show this screen.
  -v, --version  Show version.
  --season=<se>  Season of show. [Default is 01]
  -b, --bash     Prints output that can be piped to bash
  -l, --log      Enables logging
  -r, --recurse  Enables recursive finding of files
"""


import docopt
import lists
import os
import re
import strformat
import strutils
import sugar
import terminal
import zero_functional

func findYear*(input: string): string =
  ## Returns string containing year from given input
  let seq_year = input.split(".") --> filter(re"^\d{4}$" in it)
  seq_year.join()

func findExt*(input: string): string =
  ## Returns string containing file extention of given input
  let (dir, name, ext) = splitFile(input)
  return ext

func findMovie*(input: string): string =
  ## Returns string containing movie name of given input
  let file = splitPath(input)
  let year = findYear(file.tail)

  let seq_unsplit_title = file.tail.split(year) --> sub(0, 0)
  let seq_title = seq_unsplit_title.join().replace(".", " ")

  let ext = findExt(file.tail)

  fmt"{seq_title.join()} ({year}){ext}"

func isMovie*(input: string): bool =
  ## Returns bool of movie detected
  not isEmptyOrWhitespace(findYear(input))


func findSubber*(input: string): string =
  ## Returns string containing subber of given input
  ## Returns empty string if not found
  let first_str = input.split(" ")[0].join()
  if first_str.contains("[") and first_str.contains("]"):
    return first_str
  else:
    return ""

func isSubber*(input: string): bool =
  ## Returns bool of subber detected
  not isEmptyOrWhitespace(findSubber(input))

func findEP*(input: string): string =
  ## Returns string containing episode number of given input
  let seq_ep = input.split(" ") --> filter(re"^\d{2}$" in it)
  seq_ep.join()

func isEP*(input: string): bool =
  ## Returns bool of episode number detected
  not isEmptyOrWhitespace(findEP(input))

func seasonCheck(season: string): string =
  # Ensure seasons below 10 start with 0. Example is 03
  if not isEmptyOrWhitespace(season):
    if season.len() < 2: fmt"S0{season}E"
    else: fmt"S{season}E"
  else: "S01E"

func findAnime*(input: string, season: string): string =
  ## Returns string containing anime name of given input
  let file = splitPath(input)
  let ep = findEP(file.tail)
  var fmtSeason = fmt"{seasonCheck(season)}"

  # Strips information to the right of EP
  let seq_unsplit_title = file.tail.split(ep) --> sub(0, 0)
  let rstripped_title = seq_unsplit_title.join(sep = " ")
  # Strips subber
  let seq_title = rstripped_title.split(" ") --> sub(1, ^1)
  let title = seq_title.join(sep = " ")

  let ext = findExt(file.tail)

  fmt"{title}{fmtSeason}{ep}{ext}"

func isAnime*(input: string): bool =
  ## Returns bool of anime detected
  not isEmptyOrWhitespace(findSubber(input))

func normalize*(dir: string): string =
  ## Returnes a normalized filename
  normalizedPath(dir).dup(normalizeExe)

func isDir*(input: string): bool =
  ## Returns bool of if input is a path
  findExt(input.normalize).isEmptyOrWhiteSpace

proc main() =
  # Initializes season to be blank. Can be overridden with command
  var
    file_list = initDoublyLinkedList[string]()
    season: string
    bash: bool
    recurse: bool
    fromDir: string
    toDir: string

  let args = docopt(doc, version = "Renamer 0.1")
  let path = args["<path>"]
  if args["--season"]: season = $args["--season"]
  if args["--bash"]: bash = parseBool($args["--bash"])
  if args["--recurse"]: recurse = parseBool($args["--recurse"])

  # Builds file list up to 1 directory deep. Last option is the output dir
  for i, c in path:
    if i < path.len-1:
      if not c.isDir:
        file_list.append(c.join)
      else: # Is a dir, walk and add files
        if not recurse:
          for kind, path in walkDir(c):
            if not path.isDir: file_list.append(path)
        else: # recursive
          for x in walkDirRec(c):
            if not isDir(x): file_list.append(x)
    else: toDir = c

  if not isatty(stdin):
    while true:
      try:
        file_list.append(readLine(stdin))
      except EOFError:
        break

  for i in file_list:
    if isMovie(i):
      let frm = normalize(fmt"{fromDir}{i}")
      let to = normalize(fmt"{toDir}{findMovie(i)}")
      if bash: echo fmt"""mv "{frm}" "{to}""""
      else:
        if args["--log"]: echo fmt"Moving {frm} to {to}"
        moveFile(frm, to)
    if isAnime(i):
      if isEP(i):
        let frm = normalize(fmt"{fromDir}{i}")
        let to = normalize(fmt"{toDir}{findAnime(i, season)}")
        if bash: echo fmt"""mv "{frm}" "{to}""""
        else:
          if args["--log"]: echo fmt"Moving {frm} to {to}"
          moveFile(frm, to)
      else: discard # echo "Anime Movie?"


when isMainModule:
  main()
